/* eag_demo.c */
/* just a basic demo showing off a little palette rotation, and some simple bouncing sprite routines. Nothing much to look at, but could be helpful*/


#include "huc.h"

#define EAGAN_SPR 20
#define EAGAN_MAX 0x1500
#define EAGAN_SIZE 0x100
#define SNAKE_MAX 18
#define SNAKE_1   0
#define SNAKE_2   12

/*where do we want to put stuff in video ram? */
#define TILE_VRAM    0x1000
#define SPRITE_VRAM  0x6000

/*sprites*/
#incspr(orb_sprs, "orb.pcx",0,0,1,1);
#incpal(orb_pal, "orb.pcx",0,1);

#incspr(eag_sprs, "eag_logo.pcx",0,0,15,1);
#incpal(eag_pal, "eag_logo.pcx",0,1);

/*background*/
#incchr(backpic,"back.pcx",32,28);
#incpal(backpal,"back.pcx");
#incbat(backbat,"back.pcx",TILE_VRAM,32,28);

/*
 * init() does exactly what it sounds like it does. it inits our routine.
*/
init() {
  load_background(backpic, backpal, backbat, 32, 28);/*just the one background*/
  load_vram(SPRITE_VRAM, orb_sprs, 0x40);            /*Just the one sprite:)*/
  load_sprites(0x6040, eag_sprs, 15);
  set_sprpal(0, orb_pal);                            /*set up our orb palette*/
  set_sprpal(1, eag_pal);                            /*set up our eag palette*/
  init_satb();                                       /*can't do much without the sprite attribute table:)*/
  /*set_color_rgb(1, 5,5,5);                           I would use this If I wanted to write text, but I'm not*/
}/*end init function*/

/*
rot_pal will rotate the selected palette, ether up or down.
*/
rot_pal(palnum1, palnum2) int palnum1, palnum2; {

   int idx, rgbback, newcolor;

   if(palnum1 < palnum2) { /*if palnum is less than palnum 2, rotate up*/

        rgbback = get_color(palnum1);
        for(idx = palnum1; idx < palnum2; idx++) {
           newcolor = get_color(idx+1);
           set_color(idx, newcolor);
        }
        set_color(palnum2, rgbback);
   }
   else {                 /*if palnum is greater than palnum 2, rotate down*/
        rgbback = get_color(palnum1);
        for(idx = palnum1; idx > palnum2; idx--) {
           newcolor = get_color(idx-1);
           set_color(idx, newcolor);
        }
        set_color(palnum2, rgbback);

   }
}/*end rot_pal function*/

/* run_once: (local)
 *  Runs the demo:)
 */
run_once() {
      int x1[SNAKE_MAX];
      int y1[SNAKE_MAX];
      int eag_cntr;
      /* gravity, xvelocity, yvelocity (1 and 2), life, and total life (1 and 2), and an index counter*/
      int g, xv, yv, xv2, yv2, l1, tl1, l2, tl2, idx;

      eag_cntr = 0x6040;

      x1[SNAKE_1] = x1[SNAKE_2] = rand() % 200;
      y1[SNAKE_1] = y1[SNAKE_2] = rand() % 200;

      xv =  (rand() %5)-3;/*we want our orbs to go both left and right, up and down, so make sure that gravity can be positive or negative*/
      yv =  (rand() %8)-7;
      xv2 = (rand() %5)-3;
      yv2 = (rand() %8)-7;
      g = 1;              /*gravity is the same for both orb snakes, so we don't need more than one*/
      l1 = 0;
      tl1 = rand() %100;
      l2 = 0;
      tl2 = rand() % 100;



 for(;;) {
      rot_pal(255,241); /*rotate the palette for our background image*/

      /*eag_cntr += EAGAN_SIZE;*/
      if(eag_cntr > (EAGAN_MAX + 0x6040))eag_cntr = 0x6040;
      
      /*spr_set(EAGAN_SPR);
      spr_pal(1);
      spr_pri(1);
      spr_ctrl(SIZE_MAS|FLIP_MAS, SZ_32x64|NO_FLIP);
      spr_x(32);
      spr_y(32);
      spr_pattern(0x6080);
      */
      for(idx = 0; idx < SNAKE_MAX; idx++) { /*setup our sprites on the screen*/
      spr_set(idx);
      spr_pal(0);
      spr_pri(1);
      spr_ctrl(SIZE_MAS|FLIP_MAS, SZ_16x16|NO_FLIP);
      spr_x(x1[idx]);
      spr_y(y1[idx]);
      spr_pattern(SPRITE_VRAM);
      }
      satb_update();
      vsync(3);

      for(idx = SNAKE_MAX -1; idx > SNAKE_2; idx--){/*assign the values for our 1st snake*/
       x1[idx] = x1[idx - 1];
       y1[idx] = y1[idx - 1];
      }
      for(idx =  SNAKE_2-1; idx >  SNAKE_1; idx--){/*assign the values for our 2nd snake*/
       x1[idx] = x1[idx - 1];
       y1[idx] = y1[idx - 1];
      }

      /*move our two snakes*/
      x1[SNAKE_1] += xv;
      y1[SNAKE_1] += yv;
      x1[SNAKE_2] += xv2;
      y1[SNAKE_2] += yv2;
      
      /*add a little gravity*/
      yv += g;
      yv2 += g;

      /*reduce the lifespan of each orb*/
      l1 += 1;
      l2 += 1;
      
      /*bounce the 2nd snake off of the sides of the screen, somewhat realistically*/
      if(x1[SNAKE_1] > 232) {xv = -xv; x1[SNAKE_1] = 232;  yv-=yv/4;xv-=xv/4;}
      if(x1[SNAKE_1] < 2 )  {xv = -xv; x1[SNAKE_1] = 10;   yv-=yv/4;xv-=xv/4;}
      if(y1[SNAKE_1] > 200) {yv = -yv; y1[SNAKE_1] = 200;  yv-=yv/4;xv-=xv/4;}
      if(y1[SNAKE_1] < 10)  {yv = -yv; y1[SNAKE_1] = 10;   yv-=yv/4;xv-=xv/4;}
      /*if our lifespan runs out*/
      if(l1 >= tl1) {
       x1[SNAKE_1] = rand() % 256;
       y1[SNAKE_1] = rand() % 10;
       xv = (rand() % 26) - 13;
       yv = (rand() % 26) - 13;
       l1 = 0;
       tl1 = rand() % 240;
      }
      /*bounce our 1st snake*/
      if(x1[SNAKE_2] > 240) {xv2 = -xv2;x1[SNAKE_2] = 240;  yv2-=yv2/4;xv2-=xv2/4;}
      if(y1[SNAKE_2] > 200 ){yv2 = -yv2;y1[SNAKE_2] = 200;  yv2-=yv2/4;xv2-=xv2/4;}
      if(y1[SNAKE_2] < 2)   {yv2 = -yv2;y1[SNAKE_2] = 2;  yv2-=yv2/4;xv2-=xv2/4;}
      if(x1[SNAKE_2] < 2 )  {xv2 = -xv2;x1[SNAKE_2] = 10;  yv2-=yv2/4;xv2-=xv2/4;}
      /*if our lifespan runs out*/
      if(l2 >= tl2) {
       x1[SNAKE_2] = rand() % 256;
       y1[SNAKE_2] = rand() % 10;
       xv2 = (rand() % 26) - 13;
       yv2 = (rand() % 26) - 13;
       l2 = 0;
       tl2 = rand() % 200;
      }

  }/*end for*/

}/*end runonce*/

/* main:
 * our main loop. init everything, run our game loop, and kill it if we exit
 * god knows how that might work... maybe shutdown could have that final ending
 * that locks up the system and you have to re-boot.
 */
main() {
	init();
	run_once();
 return;
}



